@@ -197,7 +197,8 @@ class MyFrame(wx.Frame):
             self.MainIcon = None
 
 
-    def add_fitting_tab(self, event=None, modelid=None, counter=None):
+    def add_fitting_tab(self, event=None, modelid=None, counter=None,
+                        select=False):
         """ This function creates a new page inside the notebook.
             If the function is called from a menu, the modelid is 
             known by the event. If not, the modelid should be specified by 
@@ -212,6 +213,8 @@ class MyFrame(wx.Frame):
         if modelid is None:
             # Get the model id from the menu
             modelid = event.GetId()
+            # Give the new page focus
+            select = True
         if counter is not None:
             # Set the tabcounter right, so the tabs are counted continuously.
             counterint = int(counter.strip().strip(":").strip("#"))
@@ -229,7 +232,13 @@ class MyFrame(wx.Frame):
         Newtab = page.FittingPanel(self, counter, modelid, active_parms,
                                    self.tau)
         #self.Freeze()
-        self.notebook.AddPage(Newtab, counter+model, select=True)
+        self.notebook.AddPage(Newtab, counter+model, select=select)
+        if select:
+            # A hack to have the last page displayed in the tab menu:
+            Npag = self.notebook.GetPageCount()
+            for i in range(int(Npag)):
+                self.notebook.SetSelection(i)
+
         #self.Thaw()
         self.tabcounter = self.tabcounter + 1
         # Enable the "Current" Menu
@@ -243,18 +252,15 @@ class MyFrame(wx.Frame):
         # window is open.
         # Find Tool Statistics
         # Get open tools
-        toolkeys = self.ToolsOpen.keys()
-        for key in toolkeys:
-            tool = self.ToolsOpen[key]
-            try:
-                if tool.MyName=="STATISTICS":
-                    # Call the function properly.
-                    tool.OnPageChanged(Newtab)
-            except:
-                pass
-        #
-        #######
-        #
+        #toolkeys = self.ToolsOpen.keys()
+        #for key in toolkeys:
+        #    tool = self.ToolsOpen[key]
+        #    try:
+        #        if tool.MyName=="STATISTICS":
+        #            # Call the function properly.
+        #            tool.OnPageChanged(Newtab)
+        #    except:
+        #        pass
         return Newtab
 
 
@@ -461,6 +467,20 @@ class MyFrame(wx.Frame):
             # Try to import a selected .txt file
             try:
                 NewModel.GetCode( os.path.join(dirname, filename) )
+            except NameError:
+                # sympy is probably not installed
+                # Warn the user
+                text = ("SymPy not found.\n"+
+                        "In order to import user defined model\n"+
+                        "functions, please install Sympy\n"+
+                        "version 0.7.2 or higher.\nhttp://sympy.org/")
+                if platform.system().lower() == 'linux':
+                    text += ("\nSymPy is included in the package:\n"+
+                             "   'python-sympy'")
+                dlg = wx.MessageDialog(None, text, 'SymPy not found', 
+                                wx.OK | wx.ICON_EXCLAMATION)
+                dlg.ShowModal()
+                return
             except:
                 # The file does not seem to be what it seems to be.
                 info = sys.exc_info()
@@ -470,14 +490,16 @@ class MyFrame(wx.Frame):
                 errstr += str(info[1])+"\n"
                 for tb_item in traceback.format_tb(info[2]):
                     errstr += tb_item
-                wx.MessageDialog(self, errstr, "Error", 
+                dlg = wx.MessageDialog(self, errstr, "Error", 
                     style=wx.ICON_ERROR|wx.OK|wx.STAY_ON_TOP)
+                dlg.ShowModal()
                 del NewModel
                 return
             # Test the code for sympy compatibility.
             # If you write your own parser, this might be easier.
             try:
                 NewModel.TestFunction()
+
             except:
                 # This means that the imported model file could be
                 # contaminated. Ask the user how to proceed.
@@ -505,7 +527,14 @@ class MyFrame(wx.Frame):
             
 
     def OnClearSession(self,e=None,clearmodels=False):
-        """Open a previously saved session. """
+        """
+            Clear the entire session
+        
+            Returns:
+            "abort" if the user did not want to clear the session
+            "clear" if the session has been cleared and the user did
+                    or did not save the session
+        """
         numtabs = self.notebook.GetPageCount()
         # Ask, if user wants to save current session.
         if numtabs > 0:
@@ -521,7 +550,9 @@ class MyFrame(wx.Frame):
             if result == wx.ID_CANCEL:
                 return "abort"      # stop this function - do nothing.
             elif result == wx.ID_YES:
-                self.OnSaveSession()
+                filename = self.OnSaveSession()
+                if filename is None:
+                    return "abort"
             elif result == wx.ID_NO:
                 pass
         # Delete all the pages
@@ -547,6 +578,7 @@ class MyFrame(wx.Frame):
             menu=self.modelmenudict["User"]
             for item in  menu.GetMenuItems():
                 menu.RemoveItem(item)
+        return "clear"
 
 
     def OnCommSession(self,e=None):
@@ -601,6 +633,10 @@ class MyFrame(wx.Frame):
         
 
     def OnExit(self,e=None):
+        """
+            Kindly asks the user if he wants to save the session and
+            then exit the program.
+        """
         numtabs = self.notebook.GetPageCount()
         # Ask, if user wants to save current session.
         if numtabs > 0:
@@ -613,7 +649,11 @@ class MyFrame(wx.Frame):
             if result == wx.ID_CANCEL:
                 return # stop this function - do nothing.
             elif result == wx.ID_YES:
-                self.OnSaveSession()
+                filename = self.OnSaveSession()
+                if filename is None:
+                    # Do not exit. The user pressed abort in the session
+                    # saving dialog.
+                    return
         # Exit the Program
         self.Destroy()
 
@@ -626,24 +666,31 @@ class MyFrame(wx.Frame):
 
 
 
-    def OnFNBPageChanged(self,e=None, Page=None):
+    def OnFNBPageChanged(self, e=None, Page=None, trigger=None):
         """ Called, when 
             - Page focus switches to another Page
             - Page with focus changes significantly:
                 - experimental data is loaded
                 - weighted fit was done
+            - trigger is a string. For more information read the
+              doc strings of the `tools` submodule.
         """
+        
+        if (e is not None and
+            e.GetEventType()==fnb.EVT_FLATNOTEBOOK_PAGE_CHANGED.typeId
+            and trigger is None):
+            trigger = "tab_browse"
         # Get the Page
         if Page is None:
             Page = self.notebook.GetCurrentPage()
         keys = self.ToolsOpen.keys()
         for key in keys:
             # Update the information
-            self.ToolsOpen[key].OnPageChanged(Page)
+            self.ToolsOpen[key].OnPageChanged(Page, trigger=trigger)
         # parameter range selection tool for page.
         if self.RangeSelector is not None:
             try:
-                self.RangeSelector.OnPageChanged(Page)
+                self.RangeSelector.OnPageChanged(Page, trigger=trigger)
             except:
                 pass
         # Bugfix-workaround for mac:
@@ -819,7 +866,13 @@ class MyFrame(wx.Frame):
 
 
     def ImportData(self, Page, dataexp, trace, curvetype="",
-                   filename="", curveid="", run=""):
+                   filename="", curveid="", run="", trigger=None):
+        """
+            Import data into the current page.
+            
+            `trigger` is passed to PlotAll. For more info see the
+            submodule `tools`.
+        """
         CurPage = Page
         # Import traces. Traces are usually put into a list, even if there
         # is only one trace. The reason is, that for cross correlation, we 
@@ -830,11 +883,11 @@ class MyFrame(wx.Frame):
         # doing this in order to keep data types clean.
         if curvetype[0:2] == "CC":
             # For cross correlation, the trace has two components
-            CurPage.IsCrossCorrelation = True
+            CurPage.SetCorrelationType(True, init=True)
             CurPage.tracecc = trace
             CurPage.trace = None
         else:
-            CurPage.IsCrossCorrelation = False
+            CurPage.SetCorrelationType(False, init=True)
             CurPage.tracecc = None
             if trace is not None:
                 CurPage.trace = trace
@@ -848,7 +901,7 @@ class MyFrame(wx.Frame):
         # It might be possible, that we want the channels to be
         # fixed to some interval. This is the case if the 
         # checkbox on the "Channel selection" dialog is checked.
-        self.OnFNBPageChanged()
+        #self.OnFNBPageChanged()
         # Enable Fitting Button
         CurPage.Fit_enable_fitting()
         # Set new tabtitle value and strip leading or trailing
@@ -859,11 +912,11 @@ class MyFrame(wx.Frame):
             title = "{} id{:03d}-{}".format(filename, int(curveid), curvetype)
         CurPage.tabtitle.SetValue(title.strip())
         # Plot everything
-        CurPage.PlotAll()
+        CurPage.PlotAll(trigger=trigger)
         # Call this function to allow the "Channel Selection" window that
         # might be open to update itself.
         # We are aware of the fact, that we just did that
-        self.OnFNBPageChanged()
+        #self.OnFNBPageChanged()
 
 
     def OnLatexCheck(self,e):
@@ -897,7 +950,7 @@ class MyFrame(wx.Frame):
                         "  - latex\n"+
                         "  - dvipng\n"+
                         "  - ghostscript\n"+
-                        "  - texlive-science\n"+
+                        "  - texlive-latex-base\n"+
                         "  - texlive-math-extra\n")
             dlg = wx.MessageDialog(None, text, 'Latex not found', 
                             wx.OK | wx.ICON_EXCLAMATION)
@@ -1109,13 +1162,14 @@ class MyFrame(wx.Frame):
             # Fill Page with data
             self.ImportData(CurPage, Correlation[i], Trace[i],
                             curvetype=Type[i], filename=Filename[i],
-                            curveid=str(Curveid[i]), run=str(Run[i]))
+                            curveid=str(Curveid[i]), run=str(Run[i]),
+                            trigger="page_add_batch")
             # Let the user abort, if he wants to:
             # We want to do this here before an empty page is added
             # to the notebok.
             if dlg.Update(i+1, "Loading pages...")[0] == False:
-                dlg.Destroy()
-                return
+                break
+        self.OnFNBPageChanged(trigger="page_add_finalize")
         # If the user did not select curves but chose a model, destroy
         # the dialog.
         dlg.Destroy()
@@ -1130,16 +1184,13 @@ class MyFrame(wx.Frame):
         # This will also ask, if user wants to save the current session.
         clear = self.OnClearSession(clearmodels=True)
         if clear == "abort":
-            # User pressed abort when he was asked if he wants to save the
-            # session.
+            # User pressed abort when he was asked if he wants to save
+            # the session. Therefore, we cannot open a new session.
             return "abort"
         Infodict, self.dirname, filename = \
          opf.OpenSession(self, self.dirname, sessionfile=sessionfile)
         # Check, if a file has been opened
         if filename is not None:
-            # Reset all Pages. We already gave the user the possibility to
-            # save his session.
-            # self.OnClearSession()
             self.filename = filename
             self.SetTitleFCS(self.filename)
             ## Background traces
@@ -1178,9 +1229,8 @@ class MyFrame(wx.Frame):
                 # the page later.
                 counter = Infodict["Parameters"][i][0]
                 modelid = Infodict["Parameters"][i][1]
-                self.add_fitting_tab(modelid=modelid, counter=counter)
-                # Get New Page, so we can add our stuff.
-                Newtab = self.notebook.GetCurrentPage()
+                Newtab = self.add_fitting_tab(modelid=modelid,
+                                              counter=counter)
                 # Add experimental Data
                 # Import dataexp:
                 number = counter.strip().strip(":").strip("#")
@@ -1205,7 +1255,8 @@ class MyFrame(wx.Frame):
                     for wkey in wkeys:
                         WeightKinds += [wkey]
                     Newtab.Fitbox[1].SetItems(WeightKinds)
-                self.UnpackParameters(Infodict["Parameters"][i], Newtab)
+                self.UnpackParameters(Infodict["Parameters"][i], Newtab,
+                                      init=True)
                 # Supplementary data
                 try:
                     Sups = Infodict["Supplements"][pageid]
@@ -1244,7 +1295,7 @@ class MyFrame(wx.Frame):
                     else:
                         Newtab.tracecc = trace
                 # Plot everything
-                Newtab.PlotAll()
+                Newtab.PlotAll(trigger="page_add_batch")
             # Set Session Comment
             try:
                 self.SessionComment = Infodict["Comments"]["Session"]
@@ -1257,7 +1308,7 @@ class MyFrame(wx.Frame):
             if self.notebook.GetPageCount() > 0:
                 # Enable the "Current" Menu
                 self.EnableToolCurrent(True)
-                self.OnFNBPageChanged()
+                self.OnFNBPageChanged(trigger="page_add_finalize")
             else:
                 # There are no pages in the session.
                 # Disable some menus and close some dialogs
@@ -1295,7 +1346,13 @@ class MyFrame(wx.Frame):
 
 
     def OnSaveSession(self,e=None):
-        """Save a session for later continuation."""
+        """ 
+            Save a session to a session file
+        
+            Returns:
+            - the filename of the session if it was saved
+            - None, if the user canceled the action
+        """
         # Parameters are all in one dictionary:
         Infodict = dict()
         Infodict["Backgrounds"] = self.Background # Background list
@@ -1362,10 +1419,9 @@ class MyFrame(wx.Frame):
         # If no file has been selected, self.filename will be set to 'None'.
         self.dirname, self.filename = opf.SaveSession(self, self.dirname,
           Infodict)
-        #Function_parms, Function_array, Function_trace, self.Background,
-        #Preferences, Comments, ExternalFunctions, Info)
         # Set title of our window
         self.SetTitleFCS(self.filename)
+        return self.filename
 
 
     def OnShell(self, e=None):
@@ -1427,8 +1483,8 @@ class MyFrame(wx.Frame):
         # Additional parameters as of v.0.2.0
         # Splines and model function:
         # Additional parameters as of v.6.4.0
-        #self.Fitbox=[ fitbox, weightedfitdrop, fittext, fittext2, fittextvar,
-        #                fitspin, buttonfit ]
+        #self.Fitbox=[ fitbox, weightedfitdrop, fittext, fittext2, 
+        #              fittextvar, fitspin, buttonfit ]
         # Some fits like Spline have a number of knots of the spline
         # that is important for fitting. If there is a number in the
         # Dropdown, save it.
@@ -1441,7 +1497,8 @@ class MyFrame(wx.Frame):
             knots = int(knots)
         weighted = Page.weighted_fittype_id
         weights = Page.weighted_nuvar
-        Parms.append([weighted, weights, knots])
+        algorithm = Page.fit_algorithm
+        Parms.append([weighted, weights, knots, algorithm])
         # Additional parameters as of v.0.2.9
         # Which Background signal is selected?
         # The Background information is in the list *self.Background*.
@@ -1461,11 +1518,14 @@ class MyFrame(wx.Frame):
         return Parms
 
 
-    def UnpackParameters(self, Parms, Page):
+    def UnpackParameters(self, Parms, Page, init=False):
         """ Apply the given parameters to the Page in question.
             This function contains several *len(Parms) >= X* statements.
             These are used for opening sessions that were saved using
             earlier versions of PyCorrFit.
+            The `init` variable can be set if fundamental changes
+            are made (loading data). This e.g. might change the type
+            (Autocorrelation/Cross-Correlation) of the page.
         """
         modelid = Parms[1]
         if Page.modelid != modelid:
@@ -1508,9 +1568,13 @@ class MyFrame(wx.Frame):
             if len(Parms[5]) == 2:
                 [weighted, weights] = Parms[5]
                 knots = None
-            else:
+            elif len(Parms[5]) == 3:
                 # We have knots as of v. 0.6.5
                 [weighted, weights, knots] = Parms[5]
+            else:
+                # We have different fitting algorithms as of v. 0.8.3
+                [weighted, weights, knots, algorithm] = Parms[5]
+                Page.fit_algorithm = algorithm
             if knots is not None:
                 # This is done with apply_paramters_reverse:
                 #       text = Page.Fitbox[1].GetValue()
@@ -1549,7 +1613,7 @@ class MyFrame(wx.Frame):
                 Page.OnAmplitudeCheck("init")
         # Set if Newtab is of type cross-correlation:
         if len(Parms) >= 8:
-            Page.IsCrossCorrelation = Parms[7]
+            Page.SetCorrelationType(Parms[7], init)
         if len(Parms) >= 9:
             # New feature in 0.7.8 includes normalization to a fitting
             # parameter.
