@@ -28,11 +28,6 @@
     You should have received a copy of the GNU General Public License 
     along with this program. If not, see <http://www.gnu.org/licenses/>.
 """
-# Use DEMO for contrast-rich screenshots.
-# This enlarges axis text and draws black lines instead of grey ones.
-DEMO = False
-
-
 import wx                               # GUI interface wxPython
 from wx.lib.agw import floatspin        # Float numbers in spin fields
 import wx.lib.plot as plot              # Plotting in wxPython
@@ -41,7 +36,7 @@ import numpy as np                      # NumPy
 import sys                              # System stuff
 
 import edclasses                    # Cool stuf like better floatspin
-import leastsquaresfit as fit       # For fitting
+import fitting as fit       # For fitting
 import models as mdls
 import tools
 
@@ -87,8 +82,9 @@ class FittingPanel(wx.Panel):
         self.resid = None        # Residuals
         self.data4weight = None  # Data used for weight calculation 
         # Fitting:
-        #self.Fitbox=[ fitbox, weightedfitdrop, fittext, fittext2, fittextvar,
-        #                fitspin, buttonfit ]
+        #self.Fitbox = [ fitbox, weightedfitdrop, fittext, fittext2,
+        #                fittextvar, fitspin, buttonfit, textalg,
+        #                self.AlgorithmDropdown]
         # chi squared - is also an indicator, if something had been fitted
         self.FitKnots = 5 # number of knots for spline fit or similiars
         self.chi2 = None
@@ -96,9 +92,10 @@ class FittingPanel(wx.Panel):
         self.weights_used_for_fitting = None # weights used for fitting
         self.weights_used_for_plotting = None # weights used for plotting
         self.weights_plot_fill_area = None # weight area in plot
-        self.weighted_fittype_id = None # integer (drop down item)
+        self.weighted_fittype_id = 0 # integer (drop down item)
         self.weighted_fittype = "Unknown" # type of fit used
-        self.weighted_nuvar = None # bins for std-dev. (left and rigth)
+        self.weighted_nuvar = 3 # bins for std-dev. (left and rigth)
+        self.fit_algorithm ="Lev-Mar" # Least squares min. is standard
         # dictionary for alternative variances from e.g. averaging
         self.external_std_weights = dict()
         # Errors of fit dictionary
@@ -170,11 +167,13 @@ class FittingPanel(wx.Panel):
         self.calculate_corr()
         # Draw the settings section
         self.settings()
+        # Load default values
+        self.apply_parameters_reverse()
         # Upper Plot for plotting of Correlation Function
         self.canvascorr = plot.PlotCanvas(self.spcanvas)
         self.canvascorr.setLogScale((True, False))  
         self.canvascorr.SetEnableZoom(True)
-        self.PlotAll()
+        self.PlotAll(event="init", trigger="tab_init")
         self.canvascorr.SetSize((canvasx, cupsizey))
         # Lower Plot for plotting of the residuals
         self.canvaserr = plot.PlotCanvas(self.spcanvas)
@@ -185,14 +184,6 @@ class FittingPanel(wx.Panel):
                                         cupsizey)
         self.sp.SplitVertically(self.panelsettings, self.spcanvas,
                                 self.sizepanelx)
-        ## Check out the DEMO option and make change the plot:
-        try:
-            if DEMO == True:
-                self.canvascorr.SetFontSizeAxis(16)
-                self.canvaserr.SetFontSizeAxis(16)
-        except:
-            # Don't raise any unnecessary erros
-            pass
         # Bind resizing to resizing function.
         wx.EVT_SIZE(self, self.OnSize)
 
@@ -230,6 +221,10 @@ class FittingPanel(wx.Panel):
             Knots = self.Fitbox[1].GetValue()
             Knots = filter(lambda x: x.isdigit(), Knots)
             self.FitKnots = int(Knots)
+        # Fitting algorithm
+        keys, items = fit.GetAlgorithmStringList()
+        idalg = self.AlgorithmDropdown.GetSelection()
+        self.fit_algorithm = keys[idalg]
         # If parameters have been changed because of the check_parms
         # function, write them back.
         self.apply_parameters_reverse()
@@ -262,6 +257,10 @@ class FittingPanel(wx.Panel):
         List[1] = "Spline ("+str(self.FitKnots)+" knots)"
         self.Fitbox[1].SetItems(List)
         self.Fitbox[1].SetSelection(idf)
+        # Fitting algorithm
+        keys, items = fit.GetAlgorithmStringList()
+        idalg = keys.index(self.fit_algorithm)
+        self.AlgorithmDropdown.SetSelection(idalg)
 
 
     def calculate_corr(self):
@@ -350,12 +349,6 @@ class FittingPanel(wx.Panel):
                 self.startcrop = 0
             else:
                 self.tau = 1*self.taufull[self.startcrop:self.endcrop]
-        ## ## Channel selection
-        ## # Crops the array *self.dataexpfull* from *start* (int) to *end* (int)
-        ## # and assigns the result to *self.dataexp*. If *start* and *end* are 
-        ## # equal (or not given), *self.dataexp* will be equal to 
-        ## # *self.dataexpfull*.
-        ## self.parent.OnFNBPageChanged(e=None, Page=self)
 
 
     def CorrectDataexp(self, dataexp):
@@ -406,11 +399,14 @@ class FittingPanel(wx.Panel):
 
     def Fit_enable_fitting(self):
         """ Enable the fitting button and the weighted fit control"""
-        #self.Fitbox=[ fitbox, weightedfitdrop, fittext, fittext2, fittextvar,
-        #                fitspin, buttonfit ]
+        #self.Fitbox = [ fitbox, weightedfitdrop, fittext, fittext2,
+        #                fittextvar, fitspin, buttonfit, textalg,
+        #                self.AlgorithmDropdown]
         self.Fitbox[0].Enable()
         self.Fitbox[1].Enable()
-        self.Fitbox[-1].Enable()
+        self.Fitbox[6].Enable()
+        self.Fitbox[7].Enable()
+        self.Fitbox[8].Enable()
 
 
     def Fit_create_instance(self, noplots=False):
@@ -469,11 +465,23 @@ class FittingPanel(wx.Panel):
             self.weights_used_for_fitting = Fitting.dataweights
         self.weighted_fittype_id = idf = self.Fitbox[1].GetSelection()
         self.weighted_fittype = self.Fitbox[1].GetItems()[idf]
+        # Set fitting algorithm
+        Fitting.fit_algorithm = self.fit_algorithm
         return Fitting
 
         
-    def Fit_function(self, event=None, noplots=False):
-        """ Call the fit function. """
+    def Fit_function(self, event=None, noplots=False, trigger=None):
+        """ Calls the fit function.
+            
+            `noplots=True` prevents plotting of spline fits
+        
+            `trigger` is passed to page.PlotAll.
+                      If trigger is "fit_batch", then `noplots` is set
+                      to `True`.
+        
+        """
+        if trigger in ["fit_batch"]:
+            noplots = True
         # Make a busy cursor
         wx.BeginBusyCursor()
         # Apply parameters
@@ -483,8 +491,9 @@ class FittingPanel(wx.Panel):
         Fitting = self.Fit_create_instance(noplots)
         # Reset page counter
         self.GlobalParameterShare = list()
+        Fitting.minimize()
         try:
-            Fitting.least_square()
+            Fitting.minimize()
         except ValueError:
             # I sometimes had this on Windows. It is caused by fitting to
             # a .SIN file without selection proper channels first.
@@ -513,7 +522,7 @@ class FittingPanel(wx.Panel):
         # Update spin-control values
         self.apply_parameters_reverse()
         # Plot everthing
-        self.PlotAll()
+        self.PlotAll(trigger=trigger)
         # Return cursor to normal
         wx.EndBusyCursor()
 
@@ -733,8 +742,6 @@ class FittingPanel(wx.Panel):
             # in the tab? We choose 9: AC1-012 plus 2 whitespaces
             text = self.counter + self.tabtitle.GetValue()[-9:]
         self.parent.notebook.SetPageText(pid,text)        
-        #import IPython
-        #IPython.embed()
 
         
     def OnSetRange(self, e):
@@ -765,7 +772,7 @@ class FittingPanel(wx.Panel):
         self.sp.SetSize(size)
 
 
-    def PlotAll(self, event=None):
+    def PlotAll(self, event=None, trigger=None):
         """
         This function plots the whole correlation and residuals canvas.
         We do:
@@ -773,6 +780,15 @@ class FittingPanel(wx.Panel):
         - Background correction
         - Apply Parameters (separate function)
         - Drawing of plots
+        
+        The `event` is usually just an event from buttons or similar
+        wx objects. It can be "init", then some initial plotting is
+        done before the data is handled.
+        
+        The `trigger` is passed to `self.parent.OnFNBPageChanged` so
+        that tools can update their content accordingly. For more
+        information on triggers, have a look at the doctring of the
+        `tools` submodule.
         """
         if event == "init":
             # We use this to have the page plotted at least once before
@@ -803,21 +819,10 @@ class FittingPanel(wx.Panel):
         zerostart = self.tau[0]
         zeroend = self.tau[-1]
         datazero = [[zerostart, 0], [zeroend,0]]
-        ## Check out the DEMO option and make change the plot:
-        try:
-            if DEMO == True:
-                width = 4
-                colexp = "black"
-                colfit = "red"
-            else:
-                width = 1
-                colexp = "grey"
-                colfit = "blue"
-        except:
-            # Don't raise any unnecessary erros
-            width = 1   
-            colexp = "grey"  
-            colfit = "blue"
+        # Set plot colors
+        width = 1   
+        colexp = "grey"  
+        colfit = "blue"
         colweight = "cyan"
         lines = list()
         linezero = plot.PolyLine(datazero, colour='orange', width=width)
@@ -917,7 +922,17 @@ class FittingPanel(wx.Panel):
             PlotCorr = plot.PlotGraphics([linezero, linecorr],
                        xLabel=u'Lag time τ [ms]', yLabel=u'G(τ)')
             self.canvascorr.Draw(PlotCorr)
-        self.parent.OnFNBPageChanged()
+        self.parent.OnFNBPageChanged(trigger=trigger)
+
+
+    def SetCorrelationType(self, iscc, init=False):
+        """
+            The correlation type (AC or CC) of the page is set if data
+            is imported to the page (parent.ImportData).
+            In this case, init is `True`, else `False`.
+        """
+        if init:
+            self.IsCrossCorrelation = iscc
 
 
     def settings(self):
@@ -1043,11 +1058,20 @@ class FittingPanel(wx.Panel):
         fitsizerspin.Add(fittextvar)
         fitsizerspin.Add(fitspin)
         fitsizer.Add(fitsizerspin)
+        # Add algorithm selection
+        textalg = wx.StaticText(self.panelsettings, label="Algorithm")
+        fitsizer.Add(textalg)
+        self.AlgorithmDropdown = wx.ComboBox(self.panelsettings)
+        keys, items = fit.GetAlgorithmStringList()
+        self.AlgorithmDropdown.SetItems(items)
+        self.Bind(wx.EVT_COMBOBOX, self.apply_parameters,
+                  self.AlgorithmDropdown)
+        fitsizer.Add(self.AlgorithmDropdown)
+        self.AlgorithmDropdown.SetMaxSize(weightedfitdrop.GetSize())
         # Add button "Fit"
         buttonfit = wx.Button(self.panelsettings, label="Fit")
         self.Bind(wx.EVT_BUTTON, self.Fit_function, buttonfit)
         fitsizer.Add(buttonfit)
-        
         self.panelsettings.sizer.Add(fitsizer)
         # Squeeze everything into the sizer
         self.panelsettings.SetSizer(self.panelsettings.sizer)
@@ -1055,8 +1079,9 @@ class FittingPanel(wx.Panel):
         self.panelsettings.Layout()
         self.panelsettings.Show()
         # Make all the stuff available for everyone
-        self.Fitbox = [ fitbox, weightedfitdrop, fittext, fittext2, fittextvar,
-                        fitspin, buttonfit ]
+        self.Fitbox = [ fitbox, weightedfitdrop, fittext, fittext2,
+                        fittextvar, fitspin, buttonfit, textalg,
+                        self.AlgorithmDropdown]
         # Disable Fitting since no data has been loaded yet
         for element in self.Fitbox:
             element.Disable()
