@@ -60,6 +60,10 @@ def openFCS_Multiple(dirname, filename):
         This works with files from the Confocor2, Confocor3 (AIM) and 
         files created from the newer ZEN Software.
     """
+    ### TODO:
+    # match curves with their timestamp
+    # (actimelist and cctimelist)
+    #
     openfile = open(os.path.join(dirname, filename), 'r')
     Alldata = openfile.readlines()
     # Start progressing through the file. i is the line index.
@@ -75,6 +79,10 @@ def openFCS_Multiple(dirname, filename):
     cclist = list()     # All cross-correlation functions
     # The intensity traces
     traces = list()
+    # we use "AcquisitionTime" to match up curves
+    thistime = None
+    actimelist = list()
+    cctimelist = list()
     # The correlation curves
     ac_correlations = list()
     cc_correlations = list()
@@ -84,6 +92,8 @@ def openFCS_Multiple(dirname, filename):
             fcsset = True
             gottrace = False
         if fcsset == True:
+            if Alldata[i].partition("=")[0].strip() == "AcquisitionTime":
+                thistime = Alldata[i].partition("=")[2].strip()
             if Alldata[i].partition("=")[0].strip() == "Channel":
                 # Find out what type of correlation curve we have.
                 # Might be interesting to the user.
@@ -113,6 +123,11 @@ def openFCS_Multiple(dirname, filename):
                     # the next "FcsDataSet"-section.
                     print "Unknown channel configuration in .fcs file: "+FCStype
                     fcsset = False
+                elif FoundType[:2] == "CC":
+                    cctimelist.append(thistime)
+                elif FoundType[:2] == "AC":
+                    actimelist.append(thistime)
+
             if Alldata[i].partition("=")[0].strip() == "CountRateArray":
                 # Start importing the trace. This is a little difficult, since
                 # traces in those files are usually very large. We will bin
@@ -216,6 +231,11 @@ def openFCS_Multiple(dirname, filename):
     #          ac_correlations. Not in cc_correlations,
     #          because cross-correlations are not saved with traces.?
     #
+    # Identifiers:
+    #  actimelist: aquisition times according to aclist
+    #  cctimelist: aquisition times according to cclist
+    #
+    # Correlations:
     #  ac_correlations: AC-correlation data in list.
     #  cc_correlations: CC-correlation data in list.
     # 
@@ -233,7 +253,13 @@ def openFCS_Multiple(dirname, filename):
     curvelist = list()
     tracelist = list()
     corrlist = list()
+    
+    ### TODO:
+    # match curves with their timestamp
+    # (actimelist and cctimelist)
+    
     for i in np.arange(len(ac_correlations)):
+        # Filter curves without correlation (ignore them)
         if ac_correlations[i] is not None:
             curvelist.append(aclist[i])
             tracelist.append(1*traces[i])
@@ -241,12 +267,18 @@ def openFCS_Multiple(dirname, filename):
         else:
             if traces[i] is not None:
                 warnings.warn("File {} curve {} does not contain AC data.".format(filename, i))
+    # Overwrite traces. This way we have equal number of ac correlations
+    # and traces.
+    traces = tracelist
     ## The CC traces are more tricky:
     # Add traces to CC-correlation functions.
     # It seems reasonable, that if number of AC1,AC2 and CC are equal,
     # CC gets the traces accordingly.
-    n_ac1 = aclist.count("AC1")
-    n_ac2 = aclist.count("AC2")
+    # We take the number of ac curves from curvelist instead of aclist,
+    # because aclist may contain curves without ac data (see above).
+    # In that case, the cc traces do most-likely belong to the acs.
+    n_ac1 = curvelist.count("AC1")
+    n_ac2 = curvelist.count("AC2")
     n_cc12 = cclist.count("CC12")
     n_cc21 = cclist.count("CC21")
     if n_ac1==n_ac2==n_cc12==n_cc21>0:
@@ -390,6 +422,11 @@ def openFCS_Single(dirname, filename):
                         corr.append( (np.float(row[0]), np.float(row[1])-1) )
                     corr = np.array(corr)
                 fcscurve = False
+                
+    # Check for correlation at lag-time zero, which lead to a bug (#64)
+    # on mac OSx and potentially affects fitting.
+    if corr[0][0] == 0:
+        corr = corr[1:]
     openfile.close()
     dictionary = dict()
     dictionary["Correlation"] = [corr]
