@@ -28,6 +28,7 @@
     You should have received a copy of the GNU General Public License 
     along with this program. If not, see <http://www.gnu.org/licenses/>.
 """
+from __future__ import division
 
 import datetime 
 import wx
@@ -71,7 +72,7 @@ class Stat(wx.Frame):
         # Page - the currently active page of the notebook.
         self.Page = self.parent.notebook.GetCurrentPage()
         # Pagenumbers
-        self.PageNumbers = np.arange(self.parent.notebook.GetPageCount())
+        self.PageNumbers =range(1,1+self.parent.notebook.GetPageCount())
         ## Splitter window. left side: checkboxes
         ##                  right side: plot with parameters
         self.sp = wx.SplitterWindow(self, style=wx.SP_3DSASH)
@@ -176,14 +177,17 @@ class Stat(wx.Frame):
             wx.Frame.SetIcon(self, parent.MainIcon)
         self.Show(True)
         self.OnDropDown()
-        self.OnDropDown()
 
-    def GetListOfAllParameters(self, e=None, return_std_checked=False):
+
+    def GetListOfAllParameters(self, e=None, return_std_checked=False,
+                                page=None):
         """ Returns sorted list of parameters.
             If return_std_checked is True, then a second list with
             standart checked parameters is returned.
         """
-        self.InfoClass.CurPage = self.Page
+        if page is None:
+            page = self.Page
+        self.InfoClass.CurPage = page
         # Now that we know our Page, we may change the available
         # parameter options.
         Infodict = self.InfoClass.GetCurInfo()
@@ -208,20 +212,18 @@ class Stat(wx.Frame):
                 headparm = list()
                 bodyparm = list()
                 for parm in Infodict[key]:
-                    #parminlist = False
+                    headparm.append(parm)
                     try:
                         for fitp in Infodict["fitting"]:
                             parmname = parm[0]
                             errname = "Err "+parmname
                             if fitp[0] == errname:
-                                headparm.append(parm)
-                                #parminlist = True
                                 headparm.append(fitp)
                     except:
-                        # Maybe there was not fit...
+                        # There was not fit, the fit with "Lev-Mar"
+                        # was not good, or another fit algorithm was
+                        # used.
                         pass
-                    #if parminlist == False:
-                    headparm.append(parm)
             elif key == "fitting":
                 for fitp in Infodict[key]:
                     # We added the error data before in the parm section
@@ -260,22 +262,24 @@ class Stat(wx.Frame):
             for checkitem in checklist:
                 if item[0].count(checkitem):
                     checked[i] = False
-
         if return_std_checked:
             return Info, checked
         else:
             return Info
 
         
-    def GetListOfPlottableParms(self, e=None, return_values=False):
+    def GetListOfPlottableParms(self, e=None, return_values=False,
+                                page=None):
         """ Returns sorted list of parameters that can be plotted.
             (This means that the values are convertable to floats)
             If return_values is True, then a second list with
             the corresponding values is returned.
         """
+        if page is None:
+            page = self.Page
         if self.parent.notebook.GetPageCount() != 0:
             #Info = self.InfoClass.GetPageInfo(self.Page)
-            Info = self.GetListOfAllParameters()
+            Info = self.GetListOfAllParameters(page=page)
             #keys = Info.keys()
             #keys.sort()
             parmlist = list()
@@ -380,13 +384,16 @@ class Stat(wx.Frame):
 
 
     def OnChooseValues(self, event=None):
+        """
+            Plot the values for the parameter selected by the user.
+        """
         Info, checked = self.GetListOfAllParameters(
                                                 return_std_checked=True)
         #headcounter = 0
         #headlen = len(head)
         # We will sort the checkboxes in more than one column if there
         # are more than *maxitemsincolumn*
-        maxitemsincolumn = np.float(25)
+        maxitemsincolumn = np.float(19)
         Sizernumber = int(np.ceil(len(Info)/maxitemsincolumn))
         self.boxsizerlist = list()
         for i in np.arange(Sizernumber):
@@ -432,6 +439,7 @@ class Stat(wx.Frame):
         if self.parent.notebook.GetPageCount() == 0 or self.Page is None:
             self.canvas.Clear()
             return
+        
         # Get valid pages
         strFull = self.WXTextPages.GetValue()
         try:
@@ -457,8 +465,7 @@ class Stat(wx.Frame):
                     pages.append(Page)
         plotcurve = list()
         for page in pages:
-            self.Page = page
-            pllabel, pldata = self.GetListOfPlottableParms(
+            pllabel, pldata = self.GetListOfPlottableParms(page=page,
                                                      return_values=True)
             # Get the labels and make a plot of the parameters
             if len(pllabel)-1 >= DDselid and pllabel[DDselid] == label:
@@ -475,54 +482,72 @@ class Stat(wx.Frame):
                         plotcurve.append([x,y])
         # Prepare plotting
         self.canvas.Clear()
-        linesig = plot.PolyMarker(plotcurve, size=1.5,
-                              fillstyle=wx.TRANSPARENT, marker='circle')
+        linesig = plot.PolyMarker(plotcurve, size=1.5, marker='circle')
         plotlist = [linesig]
         # average line
-
         try:
             avg = np.average(np.array(plotcurve)[:,1])
-            maxpage =  np.max(np.array(plotcurve)[:,0])
+            maxpage =  int(np.max(np.array(plotcurve)[:,0]) +1)
+            minpage =  int(np.min(np.array(plotcurve)[:,0]) -1)
         except:
+            minpage = 0
             maxpage = 0
             self.WXavg.SetValue("-")
             self.WXsd.SetValue("-")
         else:
             # Plot data
             plotavg = [[0.5, avg], [maxpage+.5, avg]]
-            lineclear = plot.PolyLine(plotavg, colour="black",
-                                      style= wx.SHORT_DASH)
+            #lineclear = plot.PolyLine(plotavg, colour="black")
+            lineclear = plot.PolyMarker(plotavg, colour="black")
             plotlist.append(lineclear)
             # Update Text control
             self.WXavg.SetValue(str(avg))
             self.WXsd.SetValue(str(np.std(np.array(plotcurve)[:,1])))
-            
         # Draw
-        self.canvas.Draw(plot.PlotGraphics(plotlist, 
-                             xLabel='page number', 
-                             yLabel=label))
-
+        # This causes a memory leak after this function has been 
+        # called several times with the same large data set.
+        # This goes away if only linesig OR lineclear are plotted.
+        #
+        #graphics = plot.PlotGraphics(plotlist, 
+        #                             xLabel='page number', 
+        #                             yLabel=label)
+        graphics = plot.PlotGraphics([linesig], 
+                                     xLabel='page number', 
+                                     yLabel=label)
+        
         # Correctly set x-axis
         minticks = 2
-        self.canvas.SetXSpec(max(maxpage, minticks))
-        # Zoom out such that we can see the end of all curves
-        try:
-            # Something sometimes goes wrong here?
-            xcenter = np.average(np.array(plotcurve)[:,0])
-            ycenter = np.average(np.array(plotcurve)[:,1])
-            scale = (maxpage+2.)/maxpage
-            self.canvas.Zoom((xcenter,ycenter), (scale, scale))
-        except:
-            pass
-        # Redraw result
-        self.canvas.Redraw()
-                         
+        self.canvas.SetXSpec(max(maxpage-minpage, minticks))
+        self.canvas.Draw(graphics, xAxis=(minpage,maxpage))
+
         
-    def OnPageChanged(self, page):
+    def OnPageChanged(self, page, trigger=None):
+        """
+            This function is called, when something in the panel
+            changes. The variable `trigger` is used to prevent this
+            function from being executed to save stall time of the user.
+            Forr a list of possible triggers, see the doc string of
+            `tools`.
+        """
         # When parent changes
         # This is a necessary function for PyCorrFit.
         # This is stuff that should be done when the active page
         # of the notebook changes.
+        # filter unwanted triggers to improve speed
+        if trigger in ["parm_batch", "fit_batch", "page_add_batch"]:
+            return
+        elif trigger in ["tab_init"] and page is not None:
+            # Check if we have to replot for a new model
+            if self.Page.modelid == page.modelid:
+                return
+        if (trigger in ["page_add_finalize"] and 
+            self.WXTextPages.GetValue() == "1"):
+            # We probably imported data with statistics window open
+            self.PageNumbers = range(1,
+                                  1+self.parent.notebook.GetPageCount())
+            setstring = misc.parsePagenum2String(self.PageNumbers)
+            self.WXTextPages.SetValue(setstring)
+            
         #
         # Prevent this function to be run twice at once:
         #
